@@ -4,15 +4,19 @@ from typing import Optional
 
 from . import model
 from . import dot
-from . import parse
+from . import parser
 from . import dfd_dot_templates as TMPL
-
+from . import preprocessor
 
 def build(dfd_src: str, output_path: str, percent_zoom: int, bgcolor: str,
-          format: str, debug: bool) -> None:
+          format: str, debug: bool,
+          snippet_by_name: dict[str, str] = None) -> None:
     """Take a DFD source and build the final image or document"""
-    statements = parse.parse(dfd_src)
-    items_by_name = parse.check(statements)
+    dfd_src = preprocessor.preprocess(output_path, dfd_src, debug, snippet_by_name)
+
+    statements = parser.parse(dfd_src)
+    items_by_name = parser.check(statements)
+    statements = filter_items(statements)
     gen = Generator()
     text = generate_dot(gen, statements, items_by_name)
     if debug:
@@ -122,3 +126,25 @@ def generate_dot(gen: Generator,
                 gen.generate_style(style)
 
     return gen.generate_dot_text()
+
+
+def filter_items(statements: list[model.Statement]) -> list[model.Statement]:
+    # collect used items
+    connected_items = set()
+    for statement in statements:
+        match statement:
+            case model.Connection() as conn: pass
+            case _: continue
+        for point in conn.src, conn.dst:
+            connected_items.add(point)
+
+    # filter out unconnected items
+    new_statements = []
+    for statement in statements:
+        match statement:
+            case model.Item() as item:
+                if item.hidable and item.name not in connected_items:
+                    continue
+        new_statements.append(statement)
+
+    return new_statements
\ No newline at end of file
